<?php

# config
$webroot = 'webroot';

# allows to write debug messages to the console,
# using somethin like fwrite($stdErr, "hello world\n");
$stdErr = fopen('php://stderr', 'w');

# uncomment to see _SERVER array in console
# fwrite($stdErr, print_r($_SERVER, 1)."\n");

# serve icons for directory listings from custom 'icons' folder
if (substr($_SERVER['SCRIPT_NAME'], 0, 7)==='/icons/' && file_exists(substr($_SERVER['SCRIPT_NAME'], 1))){
	header('Content-Type: image/gif');
	readfile(substr($_SERVER['SCRIPT_NAME'], 1));
	exit;
}

# make sure DirectoryIndex 'index.html' is handled correctly
$uri = $_SERVER['REQUEST_URI'];
if (is_dir($webroot.$uri)){
	if (substr($uri,-1)!='/') $uri .= '/';
	if (file_exists($webroot.$uri.'index.html')){
		header('Location: '.$uri.'index.html');
		exit;
	}
}

# check if requested URL is a directory
$is_dir = is_dir($webroot.$_SERVER['SCRIPT_NAME']);

# handle files
if (!$is_dir){

	if (file_exists($webroot.$_SERVER['SCRIPT_NAME'])){

		########################################
		# your custom header-map
		########################################
		$header_map = array(
			'xml' => 'Content-Type: application/xml'
			# more filetypes/headers here...
		);

		$ext = end(explode('.', $_SERVER['SCRIPT_NAME']));
		if (isset($header_map[$ext])){
			header($header_map[$ext]);
			readfile($webroot.$_SERVER['SCRIPT_NAME']);
			exit;
		}
	}

    # no routing for other files, serve everything as-is
	return false;
}

# otherwise serve directory listing

function cmp($a, $b){
	global $sort, $mult;
	if ($a[$sort] == $b[$sort]) return strcmp($a['name'],$b['name'])*$mult; # 0
	return ($a[$sort] < $b[$sort]) ? -1*$mult : 1*$mult;
}

function serveListing(){
	global $webroot, $sort, $mult;

	$folder = $webroot.$_SERVER['SCRIPT_NAME'];
	$sort = 'name';
	$dir =  'A';
	$props = array('N'=>'name','M'=>'mod','S'=>'size','D'=>'desc'); #D
	foreach ($props as $k=>$v){
		if (isset($_GET[$k])){
			$sort = $v;
			$dir = $_GET[$k];
		}
	}
	$mult = $dir=='A'?1:-1;

	$maxlen = 23;
	$files = array();
	$folders = array();

	if (!$handle = opendir($folder)) exit;
	while (false !== ($filename = readdir($handle))){
		if ($filename{0} != '.' && $filename != 'index.php'){
			if (is_dir($folder.'/'.$filename)){
				$folders[] = array('name'=>$filename, 'mod'=>filemtime($folder.'/'.$filename));

			} else {
				$ext = end(explode('.', $filename));
				switch ($ext){
					case 'txt':
					case 'htm':
					case 'html':
						$type = 'TXT';
						$icon = 'text.gif';
						break;
					case 'jpg':
					case 'png':
					case 'gif':
					case 'bmp':
						$type = 'IMG';
						$icon = 'image.gif';
						break;
					case 'zip':
					case 'gz':
					case 'bz2':
					case 'tar':
						$type = 'ZIP';
						$icon = 'compressed.gif';
						break;
					default:
						$type = '   ';
						$icon = 'unknown.gif';
						break;
				}
				$size = filesize($folder.'/'.$filename);
				$files[] = array('name'=>$filename, 'type'=>$type, 'icon'=>$icon, 'mod'=>filemtime($folder.'/'.$filename), 'size'=>$size);
			}

			$maxlen = max($maxlen, strlen($filename));
		}
	}
	closedir($handle);

	usort($folders, 'cmp');
	usort($files, 'cmp');

?><!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">
<html>
<head>
<title>Index of <?=$_SERVER['SCRIPT_NAME']?></title>
</head>
<body>
<h1>Index of <?=$_SERVER['SCRIPT_NAME']?></h1>
<pre><img src="/icons/blank.gif" ALT="     "> <a href="?N=<?=$sort=='name'&&$dir=='A'?'D':'A'?>">Name</a><?=str_repeat(' ', $maxlen+1-4)?><a href="?M=<?=$sort=='mod'&&$dir=='A'?'D':'A'?>"">Last modified</a>       <a href="?S=<?=$sort=='size'&&$dir=='A'?'D':'A'?>">Size</a>
<hr><?

	if ($_SERVER['SCRIPT_NAME']!='/') echo '<img src="/icons/back.gif" alt="[PARENTDIR]"> <a href="/">Parent Directory</a>                               -'."\n";

	$base = $_SERVER['SCRIPT_NAME'];
	if ($base!='/') $base.='/';

	foreach ($folders as $f){
		echo '<img src="/icons/folder.gif" alt="DIR]"> <a href="'.$base.$f['name'].'">'.$f['name'].'</a>'.str_repeat(' ', $maxlen+1-strlen($f['name'])).date ("d-M-Y H:i", $f['mod']).'      -'."  \n";
	}

	foreach ($files as $f){
		$bytes = $f['size'];
		if ($bytes>1048576)
			$filesize = round($bytes / 1024 / 1024, 1).'M';
		elseif ($bytes>1024)
			$filesize = round($bytes / 1024).'K';
		else
			$filesize = $bytes;
		$filesize = str_pad($filesize, 7, ' ', STR_PAD_LEFT);
		echo '<img src="/icons/'.$f['icon'].'" alt="['.$f['type'].']"> <a href="'.$base.$f['name'].'">'.$f['name'].'</a>'.str_repeat(' ', $maxlen+1-strlen($f['name'])).date ("d-M-Y H:i", $f['mod']).$filesize."  \n";
	}

	?>
</pre><hr>
<address>LingoServer 1.0 at <?=$_SERVER['SERVER_NAME']?> Port <?=$_SERVER['SERVER_PORT']?></address>
</body></html>
<?
}

serveListing();
